#property copyright ". Index.mq4 7.0. Copyright  FXcoder, 2008-2014"
#property link      "http://fxcoder.net"
#property strict

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Red
#property indicator_width1 1

//  (   )
input string Formula = "USD";                                       // Formula (* and / only)

// ,     
input double Power = 1.0;                                           // Power

// 
input double Scale = 1.0;                                           // Scale

//  .  != Close    
input ENUM_APPLIED_PRICE AppliedPrice = PRICE_CLOSE;                // Applied price

//  ,   
input int MAPeriod = 0;                                             // MA period

// ,     
input int DiffMAPeriod = 0;                                         // Diff MA period

//  ,   
input ENUM_MA_METHOD MAMethod = MODE_SMA;                           // MA method

//          
input string IndexCalcCurrs = "AUD,CAD,CHF,EUR,GBP,JPY,NZD,USD";    // Index calculation currency set

//      100 (      )
input bool LogScale = false;                                        // Logarithmic scale

//   
input int MaxBars = 1000;                                           // Bar count limit

//        
// (      )
string IndexBaseCurr = "USD";

//  
double X[];

//numenator ()
string _n[];
int _nCount = 0;

//denominator ()
string _d[];
int _dCount = 0;

//   
string _indexCalcCurrsA[];

int start()
{
	//     
	int lastBarToCalc = IntPutInRange(Bars - IndicatorCounted() - 1, 0, INT_MAX);

	if (lastBarToCalc < 0)
		lastBarToCalc = 0;

	if (MaxBars > 0)
		lastBarToCalc = MathMin(lastBarToCalc, MaxBars - 1);

	double x, v;

	for (int i = lastBarToCalc; i >= 0; i--)
	{
		x = 1;

		// 
		for (int k = 0; k < _nCount; k ++)
		{
			v = getValue(_n[k], i);

			if (v == EMPTY_VALUE)
			{
				x = -1;
				break;
			}
			else
			{
				x *= v;
			}
		}

		//    
		if (x < 0)
		{
			X[i] = EMPTY_VALUE;
			continue;
		}

		// 
		for (int k = 0; k < _dCount; k++)
		{
			v = getValue(_d[k], i);

			if ((v == EMPTY_VALUE) || (v == 0))
			{
				x = -1;
				break;
			}
			else
			{
				x /= v;
			}
		}

		//    
		if (x < 0)
		{
			X[i] = EMPTY_VALUE;
			continue;
		}

		// 
		x = Scale * MathPow(x, Power);

		//  
		if (LogScale && (x > 0))
			x = MathLog(x) * 100.0;

		X[i] = x;
	}

	return(0);
}

int init()
{
	SetIndexStyle(0, DRAW_LINE);
	SetIndexBuffer(0, X);

	//  
	ArrayResize(_n, _nCount);
	ArrayResize(_d, _dCount);

	string f = UpperString(Trim(Formula));

	//      .
	//   ,      ,  *   .
	ushort ch = StringGetCharacter(f, 0);

	if ((ch != '*') && (ch != '/'))
		f = "*" + f;

	f = f + "*";

	//   
	f = ReplaceString(f, "[0]", Symbol());

	if (StringLen(Symbol()) == 6)
	{
		f = ReplaceString(f, "[1]", StringSubstr(Symbol(), 0, 3));
		f = ReplaceString(f, "[2]", StringSubstr(Symbol(), 3, 3));
	}

	//    
	string indexes[];
	ArrayFree(indexes);
	int indexCount = 0;

	//   (    )
	string sym = "";
	int dir = 0;
	string ns = "";
	string ds = "";
	int fLen = StringLen(f);

	for (int i = 0; i < fLen; i++)
	{
		ch = StringGetChar(f, i);

		if ((ch == '*') || (ch == '/'))
		{
			//  , 
			if (StringLen(sym) != 0)
			{
				if (dir == 1)
				{
					_nCount++;
					ArrayResize(_n, _nCount);
					_n[_nCount - 1] = sym;
					ns = ns + " [" + sym + "]";

					if (isIndex(sym))
						StringArrayAdd(indexes, TrimLeft(sym, '!'));
				}
				else if (dir == -1)
				{
					_dCount++;
					ArrayResize(_d, _dCount);
					_d[_dCount - 1] = sym;
					ds = ds + " [" + sym + "]";

					if (isIndex(sym))
						StringArrayAdd(indexes, TrimLeft(sym, '!'));
				}

				sym = "";
			}

			//   
			dir = (ch == '*') ? 1 : -1;
		}
		else
		{
			sym = sym + CharToStr((uchar)ch);
		}
	}

	//         IndexCalcCurrsA

	indexCount = ArraySize(indexes);

	if (indexCount > 0)
	{
		//   CalcCurrs    
		string calcCurrs = IndexCalcCurrs + "," + StringArrayToString(indexes, ",");

		//  
		indexCount = SplitString(calcCurrs, ",", indexes, true);
		StringArrayGroup(indexes, _indexCalcCurrsA);

	}

	IndicatorShortName(WindowExpertName() + ": " +
		(LogScale ? "100*ln(" : "") +
		(Power != 1 ? "(" : "") +
		(Scale != 1 ? DoubleToCompactString(Scale) + "*" : "") + Trim(f, '*') +
		(Power != 1 ? ")^" + DoubleToCompactString(Power) : "") +
		(LogScale ? ")" : "") +
		"," + AppliedPriceToString(AppliedPrice)
		);

	return(0);
}

double getValue(string sym, int bar)
{
	double v = EMPTY_VALUE;

	if (isIndex(sym))
	{
		/*  */
		if (StringGetChar(sym, 0) == '!')
			sym = StringSubstr(sym, 1);

		string syms[];
		int dirs[];
		int crossCount;

		if (IndexInitA(sym, IndexBaseCurr, _indexCalcCurrsA, syms, dirs, crossCount))
		{
			v = Index(syms, dirs, crossCount, AppliedPrice, MAPeriod, MAMethod, bar);

			if (DiffMAPeriod > 0 && v > 0)
			{
				double d = Index(syms, dirs, crossCount, AppliedPrice, DiffMAPeriod, MAMethod, bar);
				v = (d > 0) ? v / d : 0;
			}
		}
	}
	else
	{
		/*    */
		v = MA(sym, 0, AppliedPrice, MAPeriod, MAMethod, bar);

		if (DiffMAPeriod > 0 && v > 0)
		{
			double d = MA(sym, 0, AppliedPrice, DiffMAPeriod, MAMethod, bar);
			v = (d > 0) ? v / d : 0;
		}
	}

	return(v);
}

// ,    .  "sym"     
bool isIndex(string &sym)
{
	int len = StringLen(sym);

	if (len > 0)
	{
		int char0 = StringGetChar(sym, 0);

		//  
		if ((len == 3) && (char0 >= 'A') && (char0 <= 'Z'))
			return(true);

		//  
		return(char0 == '!');
	}

	return(false);
}

#import "gdi32.dll"
uint GetPixel(int hDC, int x, int y);
#import

int IntPutInRange(int value, int from, int to)
{
	if (to >= from)
	{
		if (value > to)
			value = to;
		else if (value < from)
			value = from;
	}

	return(value);
}

string ReplaceString(string inputString, string from, string to)
{
	string result = "";
	int inputLength = StringLen(inputString);
	int fromLength = StringLen(from);

	for (int i = 0; i < inputLength; i++)
	{
		string s = StringSubstr(inputString, i, fromLength);

		if (s == from)
		{
  			result = StringConcatenate(result, to);
  			i += fromLength - 1;
		}
		else
		{
			result = StringConcatenate(result, StringSubstr(inputString, i, 1));
		}
	}

	return(result);
}

string TrimLeft(string s, int ch = 0)
{
	if (ch == 0)
	{
		s = StringTrimLeft(s);
	}
	else
	{
		int len = StringLen(s);

		// 
		int cut = 0;

		for (int i = 0; i < len; i++)
		{
			if (StringGetChar(s, i) == ch)
				cut++;
			else
				break;
		}

		if (cut > 0)
		{
			if (cut > len - 1)
				s = "";
			else
				s = StringSubstr(s, cut);
		}
	}
	return(s);
}

string TrimRight(string s, int ch = 0)
{
	if (ch == 0)
	{
		s = StringTrimRight(s);
	}
	else
	{
		int len = StringLen(s);

		// 
		int cut = len;

		for (int i = len - 1; i >= 0; i--)
		{
			if (StringGetChar(s, i) == ch)
				cut--;
			else
				break;
		}

		if (cut != len)
		{
			if (cut == 0)
				s = "";
			else
				s = StringSubstr(s, 0, cut);
		}
	}
	return(s);
}

string Trim(string s, int ch = 0)
{
	if (ch == 0)
		return(StringTrimLeft(StringTrimRight(s)));
	else
		return(TrimLeft(TrimRight(s, ch), ch));
}

int SplitString(string s, string sep, string& parts[], bool removeEmpty = false)
{
	int count = 0;
	int sepLen = StringLen(sep);

	string part;
	while (true)
	{
		int p = StringFind(s, sep);
		if (p >= 0)
		{
			if (p == 0)
				part = "";
			else
				part = StringSubstr(s, 0, p);

			if (!removeEmpty || (Trim(part) != ""))
			{
				count++;
				ArrayResize(parts, count);
				parts[count - 1] = part;
			}

			s = StringSubstr(s, p + sepLen);
		}
		else
		{
			//  
			if (!removeEmpty || (Trim(s) != ""))
			{
				count++;
				ArrayResize(parts, count);
				parts[count - 1] = s;
			}

			break;
		}
	}

	//    
	if ((count > 0) && (parts[count - 1] == ""))
	{
		count--;
		ArrayResize(parts, count);
	}

	return(count);
}

string UpperString(string s)
{
    StringToUpper(s);
	return(s);
}

string DoubleToCompactString(double d, int digits = 8)
{
	string s = DoubleToStr(d, digits);

	//      
	if (StringFind(s, ".") != -1)
	{
		s = TrimRight(s, '0');
		s = TrimRight(s, '.');
	}

	return(s);
}

string AppliedPriceToString(int ap, bool verbose = false)
{
	if (verbose)
	{
		switch (ap)
		{
			case 0: return("Close");
			case 1: return("Open");
			case 2: return("High");
			case 3: return("Low");
			case 4: return("Median");
			case 5: return("Typical");
			case 6: return("Weighted");
			default: return("Unknown");
		}
	}
	else
	{
		switch (ap)
		{
			case 0: return("C");
			case 1: return("O");
			case 2: return("H");
			case 3: return("L");
			case 4: return("HL");
			case 5: return("HLC");
			case 6: return("HLCC");
			default: return("?");
		}
	}
}

int StringArrayGroup(string& src[], string& dest[])
{
	int count[];
	int destCount = StringArrayGroupCount(src, dest, count);
	return (destCount);
}

int StringArrayGroupCount(string& src[], string& dest[], int& count[])
{
	int srcCount = ArraySize(src);

	//  
	int destCount = 0;
	ArrayResize(dest, 0);
	ArrayResize(count, 0);

	for (int i = 0; i < srcCount; i++)
	{
		int index = StringArrayIndexOf(dest, src[i]);

		if (index >= 0)
		{
			//   
			count[index]++;
		}
		else
		{
			//     
			destCount++;
			ArrayResize(dest, destCount);
			ArrayResize(count, destCount);
			dest[destCount - 1] = src[i];
			count[destCount - 1] = 1;
		}
	}

	return (destCount);
}

int StringArrayIndexOf(string& array[], string value, int startingFrom = 0)
{
	int index = - 1;
	int count = ArraySize(array);

	for (int i = startingFrom; i < count; i++)
	{
		if (array[i] == value)
		{
			index = i;
			break;
		}
	}

	return(index);
}

int StringArrayAdd(string& arr[], string value)
{
	int c = ArraySize(arr);
	ArrayResize(arr, c + 1);
	arr[c] = value;
	return(c);
}

int IntArrayAdd(int& arr[], int value)
{
	int c = ArraySize(arr);
	ArrayResize(arr, c + 1);
	arr[c] = value;
	return(c);
}

string StringArrayToString(string& a[], string separator)
{
	string res = "";
	int c = ArraySize(a);

	for (int i = 0; i < c; i++)
	{
		if (i != 0)
			res = StringConcatenate(res, separator);

		res = res + a[i];
	}

	return(res);
}

//  
double MA(string symbol, int timeframe, int ap, int maPeriod, int maMethod, int bar)
{
	//    
	if (symbol != Symbol())
		bar = iBarShift(symbol, timeframe, Time[bar]);

	if (maPeriod > 0)
		return(iMA(symbol, timeframe, maPeriod, 0, maMethod, ap, bar));

	//   < 2,  
	double h, l, c;
	double p = 0;

	switch(ap)
	{
		case 0:
			p = iClose(symbol, timeframe, bar);
			break;

		case 1:
			p = iOpen(symbol, timeframe, bar);
			break;

		case 2:
			p = iHigh(symbol, timeframe, bar);
			break;

		case 3:
			p = iLow(symbol, timeframe, bar);
			break;

		case 4:
			h = iHigh(symbol, timeframe, bar);
			l = iLow(symbol, timeframe, bar);
			if (h != 0 && l != 0)
				p = (h + l) / 2.0;
			break;

		case 5:
			h = iHigh(symbol, timeframe, bar);
			l = iLow(symbol, timeframe, bar);
			c = iClose(symbol, timeframe, bar);
			if (h != 0 && l != 0 && c != 0)
				p = (h + l + c) / 3.0;
			break;

		case 6:
			h = iHigh(symbol, timeframe, bar);
			l = iLow(symbol, timeframe, bar);
			c = iClose(symbol, timeframe, bar);
			if (h != 0 && l != 0 && c != 0)
				p = (h + l + 2.0 * c) / 4.0;
			break;
	}

	return(p);
}

//   (  ),   
bool IndexInitA(string index, string baseCurr, string& calcCurrs[], string& syms[], int& dirs[], int& crossCount)
{
	ArrayResize(syms, 0);
	ArrayResize(dirs, 0);

	crossCount = ArraySize(calcCurrs);
	if (StringArrayIndexOf(calcCurrs, index) == -1)
		crossCount++;

	bool ok = crossCount > 0;
	ok = ok && index__getSymbols(index, baseCurr, calcCurrs, syms, dirs);

	return(ok);
}

//     ,  maPeriod=0  
// (     ,   )
double Index(string& syms[], int& dirs[], int crossCount, int ap, int maPeriod, int maMethod, int bar)
{
	int c = 0;
	double value, q;

	//   
	value = 1.0;

	int sc = ArraySize(syms);

	for (int j = 0; j < sc; j++)
	{
		q = MA(syms[j], 0, ap, maPeriod, maMethod, bar);

		if (q > 0 && q != EMPTY_VALUE)
		{
			value *= MathPow(q, 1.0 * dirs[j] / crossCount);
			c++;
		}
		else
		{
			value = 0;
			break;
		}
	}

	if (value == 0 || sc == 0)
		value = 0.0;

	return(value);
}

//           
bool index__getSymbols(string index, string baseCurr, string& currs[], string& syms[], int& dirs[])
{
	bool ok = true;

	//    
	int cc = ArraySize(currs);

	for (int i = 0; i < cc; i++)
	{
		string curr2 = currs[i];
		if (index != curr2)
		{
			if (index == baseCurr)
				index__addSymbol(baseCurr, curr2, syms, dirs);
			else if (curr2 == baseCurr)
				index__addSymbol(index, baseCurr, syms, dirs);
			else
			{
				if (!(index__addSymbol(index, baseCurr, syms, dirs)
						&& index__addSymbol(baseCurr, curr2, syms, dirs)))
				{
					ok = false;
					break;
				}
			}
		}
	}

	if (ok)
	{
		ok = ok && (ArraySize(syms) != 0);
	}
	else
	{
		ArrayResize(syms, 0);
		ArrayResize(dirs, 0);
	}

	return(ok);
}

//      
//  curr1/curr2: EUR/USD, #AA/USD, USD/_SP500, USD/AUD.
bool index__addSymbol(string curr1, string curr2, string& syms[], int& dirs[])
{
	//        
	string sym = "";
	bool isCross = true;

	// CFD  
	if (curr2 == "USD" && (StringGetChar(curr1, 0) == '#' || StringGetChar(curr1, 0) == '_'))
	{
		sym = curr1;
		isCross = false;
	}
	else if (curr1 == "USD" && (StringGetChar(curr2, 0) == '#' || StringGetChar(curr2, 0) == '_'))
	{
		sym = curr2;
		isCross = false;
	}
	else
	{
		sym = curr1 + curr2;
	}

	//      
	if (MarketInfo(sym, MODE_BID) <= 0 && isCross)
	{
		sym = curr2 + curr1;
		if (MarketInfo(sym, MODE_BID) <= 0)
			sym = "";
	}

	//  
	if (sym != "")
	{
		int si = StringArrayIndexOf(syms, sym);
		if (isCross)
		{
			if (si >= 0)
			{
				//       ,    
				if (curr1 == StringSubstr(sym, 0, 3))
					dirs[si]++;
				else
					dirs[si]--;
			}
			else
			{
				//     
				StringArrayAdd(syms, sym);

				if (curr1 == StringSubstr(sym, 0, 3))
					IntArrayAdd(dirs, 1);
				else
					IntArrayAdd(dirs, -1);
			}
		}
		else
		{
			if (si >= 0)
			{
				//       ,    
				if (curr1 == sym)
					dirs[si]++;
				else
					dirs[si]--;
			}
			else
			{
				//     
				StringArrayAdd(syms, sym);

				if (curr1 == sym)
					IntArrayAdd(dirs, 1);
				else
					IntArrayAdd(dirs, -1);
			}
		}

		return(true);
	}

	return(false);
}

// 2014-04-03 12:59:20 UTC
// MQLMake 1.22. Copyright  FXcoder, 2011-2014. http://fxcoder.net